@@ -1,30 +1,45 @@
 # Copyright (c) 2009-2012 testtools developers. See LICENSE for details.
 
 __all__ = [
-    'AfterPreprocessing',
-    'AllMatch',
-    'Annotate',
-    'MatchesAny',
-    'MatchesAll',
-    'Not',
-    ]
+    "AfterPreprocessing",
+    "AllMatch",
+    "Annotate",
+    "AnyMatch",
+    "MatchesAll",
+    "MatchesAny",
+    "Not",
+]
 
+import sys
 import types
+from collections.abc import Callable, Iterable
+from typing import Any, Generic, TypedDict, TypeVar
+
+if sys.version_info >= (3, 11):
+    from typing import Unpack
+else:
+    from typing_extensions import Unpack
 
 from ._impl import (
     Matcher,
     Mismatch,
     MismatchDecorator,
-    )
+)
+
+T = TypeVar("T")
 
 
-class MatchesAny(object):
+class MatchesAllOptions(TypedDict, total=False):
+    first_only: bool
+
+
+class MatchesAny(Matcher[T], Generic[T]):
     """Matches if any of the matchers it is created with match."""
 
-    def __init__(self, *matchers):
+    def __init__(self, *matchers: Matcher[T]) -> None:
         self.matchers = matchers
 
-    def match(self, matchee):
+    def match(self, matchee: T) -> Mismatch | None:
         results = []
         for matcher in self.matchers:
             mismatch = matcher.match(matchee)
@@ -33,15 +48,18 @@ class MatchesAny(object):
             results.append(mismatch)
         return MismatchesAll(results)
 
-    def __str__(self):
-        return "MatchesAny(%s)" % ', '.join([
-            str(matcher) for matcher in self.matchers])
+    def __str__(self) -> str:
+        return "MatchesAny({})".format(
+            ", ".join([str(matcher) for matcher in self.matchers])
+        )
 
 
-class MatchesAll(object):
+class MatchesAll(Matcher[T], Generic[T]):
     """Matches if all of the matchers it is created with match."""
 
-    def __init__(self, *matchers, **options):
+    def __init__(
+        self, *matchers: Matcher[T], **options: "Unpack[MatchesAllOptions]"
+    ) -> None:
         """Construct a MatchesAll matcher.
 
         Just list the component matchers as arguments in the ``*args``
@@ -50,12 +68,12 @@ class MatchesAll(object):
         reported.
         """
         self.matchers = matchers
-        self.first_only = options.get('first_only', False)
+        self.first_only = options.get("first_only", False)
 
-    def __str__(self):
-        return 'MatchesAll(%s)' % ', '.join(map(str, self.matchers))
+    def __str__(self) -> str:
+        return "MatchesAll({})".format(", ".join(map(str, self.matchers)))
 
-    def match(self, matchee):
+    def match(self, matchee: T) -> Mismatch | None:
         results = []
         for matcher in self.matchers:
             mismatch = matcher.match(matchee)
@@ -72,11 +90,11 @@ class MatchesAll(object):
 class MismatchesAll(Mismatch):
     """A mismatch with many child mismatches."""
 
-    def __init__(self, mismatches, wrap=True):
-        self.mismatches = mismatches
+    def __init__(self, mismatches: "Iterable[Mismatch]", wrap: bool = True) -> None:
+        self.mismatches = list(mismatches)
         self._wrap = wrap
 
-    def describe(self):
+    def describe(self) -> str:
         descriptions = []
         if self._wrap:
             descriptions = ["Differences: ["]
@@ -84,19 +102,19 @@ class MismatchesAll(Mismatch):
             descriptions.append(mismatch.describe())
         if self._wrap:
             descriptions.append("]")
-        return '\n'.join(descriptions)
+        return "\n".join(descriptions)
 
 
-class Not(object):
+class Not(Matcher[T], Generic[T]):
     """Inverts a matcher."""
 
-    def __init__(self, matcher):
+    def __init__(self, matcher: Matcher[T]) -> None:
         self.matcher = matcher
 
-    def __str__(self):
-        return 'Not(%s)' % (self.matcher,)
+    def __str__(self) -> str:
+        return f"Not({self.matcher})"
 
-    def match(self, other):
+    def match(self, other: T) -> Mismatch | None:
         mismatch = self.matcher.match(other)
         if mismatch is None:
             return MatchedUnexpectedly(self.matcher, other)
@@ -104,69 +122,72 @@ class Not(object):
             return None
 
 
-class MatchedUnexpectedly(Mismatch):
+class MatchedUnexpectedly(Mismatch, Generic[T]):
     """A thing matched when it wasn't supposed to."""
 
-    def __init__(self, matcher, other):
+    def __init__(self, matcher: Matcher[T], other: T) -> None:
         self.matcher = matcher
         self.other = other
 
-    def describe(self):
-        return "%r matches %s" % (self.other, self.matcher)
+    def describe(self) -> str:
+        return f"{self.other!r} matches {self.matcher}"
 
 
-class Annotate(object):
+class Annotate(Matcher[T], Generic[T]):
     """Annotates a matcher with a descriptive string.
 
     Mismatches are then described as '<mismatch>: <annotation>'.
     """
 
-    def __init__(self, annotation, matcher):
+    def __init__(self, annotation: str, matcher: Matcher[T]) -> None:
         self.annotation = annotation
         self.matcher = matcher
 
     @classmethod
-    def if_message(cls, annotation, matcher):
+    def if_message(cls, annotation: str, matcher: Matcher[T]) -> Matcher[T]:
         """Annotate ``matcher`` only if ``annotation`` is non-empty."""
         if not annotation:
             return matcher
         return cls(annotation, matcher)
 
-    def __str__(self):
-        return 'Annotate(%r, %s)' % (self.annotation, self.matcher)
+    def __str__(self) -> str:
+        return f"Annotate({self.annotation!r}, {self.matcher})"
 
-    def match(self, other):
+    def match(self, other: T) -> Mismatch | None:
         mismatch = self.matcher.match(other)
         if mismatch is not None:
             return AnnotatedMismatch(self.annotation, mismatch)
+        return None
 
 
 class PostfixedMismatch(MismatchDecorator):
     """A mismatch annotated with a descriptive string."""
 
-    def __init__(self, annotation, mismatch):
-        super(PostfixedMismatch, self).__init__(mismatch)
+    def __init__(self, annotation: str, mismatch: Mismatch) -> None:
+        super().__init__(mismatch)
         self.annotation = annotation
         self.mismatch = mismatch
 
-    def describe(self):
-        return '%s: %s' % (self.original.describe(), self.annotation)
+    def describe(self) -> str:
+        return f"{self.original.describe()}: {self.annotation}"
 
 
 AnnotatedMismatch = PostfixedMismatch
 
 
 class PrefixedMismatch(MismatchDecorator):
-
-    def __init__(self, prefix, mismatch):
-        super(PrefixedMismatch, self).__init__(mismatch)
+    def __init__(self, prefix: str, mismatch: Mismatch) -> None:
+        super().__init__(mismatch)
         self.prefix = prefix
 
-    def describe(self):
-        return '%s: %s' % (self.prefix, self.original.describe())
+    def describe(self) -> str:
+        return f"{self.prefix}: {self.original.describe()}"
 
 
-class AfterPreprocessing(object):
+U = TypeVar("U")
+
+
+class AfterPreprocessing(Matcher[T], Generic[T, U]):
     """Matches if the value matches after passing through a function.
 
     This can be used to aid in creating trivial matchers as functions, for
@@ -178,7 +199,12 @@ class AfterPreprocessing(object):
           return AfterPreprocessing(_read, Equals(content))
     """
 
-    def __init__(self, preprocessor, matcher, annotate=True):
+    def __init__(
+        self,
+        preprocessor: "Callable[[T], U]",
+        matcher: Matcher[U],
+        annotate: bool = True,
+    ) -> None:
         """Create an AfterPreprocessing matcher.
 
         :param preprocessor: A function called with the matchee before
@@ -192,41 +218,38 @@ class AfterPreprocessing(object):
         self.matcher = matcher
         self.annotate = annotate
 
-    def _str_preprocessor(self):
+    def _str_preprocessor(self) -> str:
         if isinstance(self.preprocessor, types.FunctionType):
-            return '<function %s>' % self.preprocessor.__name__
+            return f"<function {self.preprocessor.__name__}>"
         return str(self.preprocessor)
 
-    def __str__(self):
-        return "AfterPreprocessing(%s, %s)" % (
-            self._str_preprocessor(), self.matcher)
+    def __str__(self) -> str:
+        return f"AfterPreprocessing({self._str_preprocessor()}, {self.matcher})"
 
-    def match(self, value):
+    def match(self, value: T) -> Mismatch | None:
         after = self.preprocessor(value)
         if self.annotate:
-            matcher = Annotate(
-                "after %s on %r" % (self._str_preprocessor(), value),
-                self.matcher)
+            matcher: Matcher[U] = Annotate(
+                f"after {self._str_preprocessor()} on {value!r}", self.matcher
+            )
         else:
             matcher = self.matcher
         return matcher.match(after)
 
 
-# This is the old, deprecated. spelling of the name, kept for backwards
-# compatibility.
-AfterPreproccessing = AfterPreprocessing
+V = TypeVar("V")
 
 
-class AllMatch(object):
+class AllMatch(Matcher["Iterable[V]"], Generic[V]):
     """Matches if all provided values match the given matcher."""
 
-    def __init__(self, matcher):
+    def __init__(self, matcher: Matcher[V]) -> None:
         self.matcher = matcher
 
-    def __str__(self):
-        return 'AllMatch(%s)' % (self.matcher,)
+    def __str__(self) -> str:
+        return f"AllMatch({self.matcher})"
 
-    def match(self, values):
+    def match(self, values: "Iterable[V]") -> Mismatch | None:
         mismatches = []
         for value in values:
             mismatch = self.matcher.match(value)
@@ -234,18 +257,19 @@ class AllMatch(object):
                 mismatches.append(mismatch)
         if mismatches:
             return MismatchesAll(mismatches)
+        return None
 
 
-class AnyMatch(object):
+class AnyMatch(Matcher["Iterable[V]"], Generic[V]):
     """Matches if any of the provided values match the given matcher."""
 
-    def __init__(self, matcher):
+    def __init__(self, matcher: Matcher[V]) -> None:
         self.matcher = matcher
 
-    def __str__(self):
-        return 'AnyMatch(%s)' % (self.matcher,)
+    def __str__(self) -> str:
+        return f"AnyMatch({self.matcher})"
 
-    def match(self, values):
+    def match(self, values: "Iterable[V]") -> Mismatch | None:
         mismatches = []
         for value in values:
             mismatch = self.matcher.match(value)
@@ -256,7 +280,7 @@ class AnyMatch(object):
         return MismatchesAll(mismatches)
 
 
-class MatchesPredicate(Matcher):
+class MatchesPredicate(Matcher[T], Generic[T]):
     """Match if a given function returns True.
 
     It is reasonably common to want to make a very simple matcher based on a
@@ -268,7 +292,7 @@ class MatchesPredicate(Matcher):
       self.assertThat(4, IsEven)
     """
 
-    def __init__(self, predicate, message):
+    def __init__(self, predicate: "Callable[[T], bool]", message: str) -> None:
         """Create a ``MatchesPredicate`` matcher.
 
         :param predicate: A function that takes a single argument and returns
@@ -280,16 +304,18 @@ class MatchesPredicate(Matcher):
         self.predicate = predicate
         self.message = message
 
-    def __str__(self):
-        return '%s(%r, %r)' % (
-            self.__class__.__name__, self.predicate, self.message)
+    def __str__(self) -> str:
+        return f"{self.__class__.__name__}({self.predicate!r}, {self.message!r})"
 
-    def match(self, x):
+    def match(self, x: T) -> Mismatch | None:
         if not self.predicate(x):
             return Mismatch(self.message % x)
+        return None
 
 
-def MatchesPredicateWithParams(predicate, message, name=None):
+def MatchesPredicateWithParams(
+    predicate: "Callable[..., bool]", message: str, name: str | None = None
+) -> "Callable[..., _MatchesPredicateWithParams[object]]":
     """Match if a given parameterised function returns True.
 
     It is reasonably common to want to make a very simple matcher based on a
@@ -298,6 +324,7 @@ def MatchesPredicateWithParams(predicate, message, name=None):
 
       HasLength = MatchesPredicate(
           lambda x, y: len(x) == y, 'len({0}) is not {1}')
+      # This assertion will fail, as 'len([1, 2]) == 3' is False.
       self.assertThat([1, 2], HasLength(3))
 
     Note that unlike MatchesPredicate MatchesPredicateWithParams returns a
@@ -313,24 +340,37 @@ def MatchesPredicateWithParams(predicate, message, name=None):
     :param message: A format string for describing mis-matches.
     :param name: Optional replacement name for the matcher.
     """
-    def construct_matcher(*args, **kwargs):
-        return _MatchesPredicateWithParams(
-            predicate, message, name, *args, **kwargs)
+
+    def construct_matcher(
+        *args: Any, **kwargs: Any
+    ) -> _MatchesPredicateWithParams[object]:
+        return _MatchesPredicateWithParams(predicate, message, name, *args, **kwargs)
+
     return construct_matcher
 
 
-class _MatchesPredicateWithParams(Matcher):
+class _MatchesPredicateWithParams(Matcher[T], Generic[T]):
+    args: "tuple[Any, ...]"
+    kwargs: "dict[str, Any]"
 
-    def __init__(self, predicate, message, name, *args, **kwargs):
+    def __init__(
+        self,
+        predicate: "Callable[..., bool]",
+        message: str,
+        name: str | None,
+        *args: Any,
+        **kwargs: Any,
+    ) -> None:
         """Create a ``MatchesPredicateWithParams`` matcher.
 
         :param predicate: A function that takes an object to match and
-            additional params as given in *args and **kwargs. The result of the
-            function will be interpreted as a boolean to determine a match.
+            additional params as given in ``*args`` and ``**kwargs``. The
+            result of the function will be interpreted as a boolean to
+            determine a match.
         :param message: A message to describe a mismatch.  It will be formatted
             with .format() and be given a tuple containing whatever was passed
-            to ``match()`` + *args in *args, and whatever was passed to
-            **kwargs as its **kwargs.
+            to ``match()`` + ``*args`` in ``*args``, and whatever was passed to
+            ``**kwargs`` as its ``**kwargs``.
 
             For instance, to format a single parameter::
 
@@ -348,18 +388,17 @@ class _MatchesPredicateWithParams(Matcher):
         self.args = args
         self.kwargs = kwargs
 
-    def __str__(self):
-        args = [str(arg) for arg in self.args]
-        kwargs = ["%s=%s" % item for item in self.kwargs.items()]
-        args = ", ".join(args + kwargs)
+    def __str__(self) -> str:
+        args_list = [str(arg) for arg in self.args]
+        kwargs_list = ["{}={}".format(*item) for item in self.kwargs.items()]
+        args_str = ", ".join(args_list + kwargs_list)
         if self.name is None:
-            name = 'MatchesPredicateWithParams(%r, %r)' % (
-                self.predicate, self.message)
+            name = f"MatchesPredicateWithParams({self.predicate!r}, {self.message!r})"
         else:
             name = self.name
-        return '%s(%s)' % (name, args)
+        return f"{name}({args_str})"
 
-    def match(self, x):
+    def match(self, x: T) -> Mismatch | None:
         if not self.predicate(x, *self.args, **self.kwargs):
-            return Mismatch(
-                self.message.format(*((x,) + self.args), **self.kwargs))
+            return Mismatch(self.message.format(*((x, *self.args)), **self.kwargs))
+        return None
